/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2013-2018 Linagora
 *
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.development.enforcer.rules;

import org.apache.maven.enforcer.rule.api.EnforcerRuleException;
import org.apache.maven.enforcer.rule.api.EnforcerRuleHelper;
import org.apache.maven.project.MavenProject;
import org.codehaus.plexus.component.configurator.expression.ExpressionEvaluationException;

/**
 * A rule to make sure that the SCM connection set in the POM is valid.
 * <p>
 * Being valid means it is the same than the one available in the SCM
 * (SVN or Git) configuration.
 * </p>
 * <p>
 * If there is no SCM configuration in the project, this rule is ignored.
 * </p>
 */
public class ScmConnectionMustMatchRealScmUrlIfAny extends AbstractRule {

    // Not final because set by injection
    private boolean shouldIFail = false;

    @Override
    public void doExecute( EnforcerRuleHelper helper ) throws EnforcerRuleException {

        final MavenProject mavenProject;
        try {
            mavenProject = (MavenProject) helper.evaluate( "${project}" );
        } catch( ExpressionEvaluationException e ) {
            throw new EnforcerRuleException( "Unable to evaluate the current maven project", e );
        }

        final String realScmConnection = this.findScmUrlInProject( mavenProject, helper );
        final String pomScmConnection;
        try {
            pomScmConnection = String.valueOf( helper.evaluate( "${project.scm.connection}" ));
        } catch( ExpressionEvaluationException e ) {
            throw new EnforcerRuleException( "Unable to evaluate the SCM connection", e );
        }

        if( realScmConnection != null
                && ! realScmConnection.equals( pomScmConnection )) {

            final String message = String.format( "Wrong SCM connection '%s' ; '%s' was expected.", pomScmConnection, realScmConnection );
            helper.getLog().warn( message );
            if( this.shouldIFail ) {
                throw new EnforcerRuleException(message);
            }
        }
    }
}
